package at.tuwien.api.database;

import at.tuwien.api.container.image.ImageDateDto;
import at.tuwien.api.database.table.columns.ColumnTypeDto;
import at.tuwien.api.database.table.columns.concepts.ConceptDto;
import at.tuwien.api.database.table.columns.concepts.UnitDto;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Size;
import lombok.*;
import lombok.extern.jackson.Jacksonized;

@Getter
@Setter
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Jacksonized
@ToString
public class ViewColumnDto {

    @NotNull
    private Long id;

    @NotNull
    @JsonProperty("database_id")
    private Long databaseId;

    @NotNull
    @Schema(example = "0")
    @JsonProperty("ordinal_position")
    private Integer ordinalPosition;

    @NotBlank
    @Size(max = 64)
    @Schema(example = "Date")
    private String name;

    @NotBlank
    @Size(max = 64)
    @JsonProperty("internal_name")
    @Schema(example = "mdb_date")
    private String internalName;

    @Schema
    private String alias;

    @JsonProperty("date_format")
    private ImageDateDto dateFormat;

    @NotNull
    @JsonProperty("auto_generated")
    @Schema(example = "false")
    private Boolean autoGenerated;

    @JsonProperty("index_length")
    private Long indexLength;

    @JsonProperty("length")
    private Long length;

    @NotNull
    @JsonProperty("column_type")
    @Schema(example = "string")
    private ColumnTypeDto columnType;

    @Schema(example = "255")
    private Long size;

    @Schema(example = "0")
    private Long d;

    private ConceptDto concept;

    private UnitDto unit;

    @Size(max = 2048)
    @Schema(example = "Column comment")
    private String description;

    @NotNull
    @JsonProperty("is_public")
    @Schema(example = "true")
    private Boolean isPublic;

    @NotNull
    @JsonProperty("is_null_allowed")
    @Schema(example = "false")
    private Boolean isNullAllowed;

}
