package at.tuwien.api.database.table.columns;

import at.tuwien.api.container.image.ImageDateDto;
import at.tuwien.api.database.ViewDto;
import at.tuwien.api.database.table.TableDto;
import at.tuwien.api.database.table.columns.concepts.ConceptDto;
import at.tuwien.api.database.table.columns.concepts.UnitDto;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Size;
import lombok.*;
import lombok.extern.jackson.Jacksonized;

import java.math.BigDecimal;
import java.util.List;

@Getter
@Setter
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Jacksonized
@ToString
@EqualsAndHashCode(onlyExplicitlyIncluded = true)
public class ColumnDto {

    @NotNull
    private Long id;

    @NotNull
    @JsonProperty("database_id")
    private Long databaseId;

    @NotNull
    @JsonProperty("table_id")
    private Long tableId;

    @NotNull
    @Schema(example = "0")
    @JsonProperty("ordinal_position")
    private Integer ordinalPosition;

    @NotBlank
    @Size(max = 64)
    @Schema(example = "Date")
    private String name;

    @NotBlank
    @Size(max = 64)
    @JsonProperty("internal_name")
    @Schema(example = "mdb_date")
    @EqualsAndHashCode.Include
    private String internalName;

    @Schema
    private String alias;

    @JsonProperty("date_format")
    private ImageDateDto dateFormat;

    @NotNull
    @JsonProperty("auto_generated")
    @Schema(example = "false")
    private Boolean autoGenerated;

    @JsonProperty("index_length")
    private Long indexLength;

    @JsonProperty("length")
    private Long length;

    @NotNull
    @JsonProperty("column_type")
    @Schema(example = "string")
    private ColumnTypeDto columnType;

    @Schema(example = "255")
    private Long size;

    @Schema(example = "0")
    private Long d;

    @Schema(example = "34300")
    @JsonProperty("data_length")
    private Long dataLength;

    @Schema(example = "34300")
    @JsonProperty("max_data_length")
    private Long maxDataLength;

    @Schema(example = "32")
    @JsonProperty("num_rows")
    private Long numRows;

    @Schema(example = "0")
    @JsonProperty("val_min")
    private BigDecimal valMin;

    @Schema(example = "100")
    @JsonProperty("val_max")
    private BigDecimal valMax;

    @Schema(example = "45.4")
    private BigDecimal mean;

    @Schema(example = "51")
    private BigDecimal median;

    @Schema(example = "5.32")
    @JsonProperty("std_dev")
    private BigDecimal stdDev;

    private ConceptDto concept;

    private UnitDto unit;

    @Size(max = 2048)
    @Schema(example = "Column comment")
    private String description;

    @ToString.Exclude
    @JsonIgnore
    @EqualsAndHashCode.Include
    private TableDto table;

    @ToString.Exclude
    @JsonIgnore
    private transient List<ViewDto> views;

    @NotNull
    @JsonProperty("is_public")
    @Schema(example = "true")
    private Boolean isPublic;

    @NotNull
    @JsonProperty("is_null_allowed")
    @Schema(example = "false")
    private Boolean isNullAllowed;

    @Parameter(description = "enum values, only considered when type = ENUM")
    private List<String> enums;

    @Parameter(description = "enum values, only considered when type = ENUM")
    private List<String> sets;

}
